@@ -4,11 +4,10 @@ use 5.008008;
 use strict;
 use warnings;
 
-use Scalar::Util qw(openhandle);
 use base 'Exporter';
 our @EXPORT_OK = our @EXPORT = qw( hash2xml );
 
-our $VERSION = '0.05';
+our $VERSION = '0.06';
 
 require XSLoader;
 XSLoader::load('XML::Hash::XS', $VERSION);
@@ -19,18 +18,20 @@ sub hash2xml {
     $options{root}     ||= 'root';
     $options{version}  ||= '1.0';
     $options{encoding} ||= 'utf-8';
-    $options{indent}     = $options{indent} ? 1 : 0;
+    $options{indent}   //= 0;
+    $options{canonical}  = $options{canonical} ? 1 : 0;
+    $options{use_attr}   = $options{use_attr}  ? 1 : 0;
 
     my $output = $options{output} || 'string';
 
     if ( $output eq 'string' ) {
-        _hash2xml2string( $hash, @options{qw( root version encoding indent )} );
+        _hash2xml2string( $hash, @options{qw( root version encoding indent canonical use_attr)} );
     }
-    elsif ( my $fh = openhandle($output) ) {
-        _hash2xml2fh( $fh, $hash, @options{qw( root version encoding indent )} );
+    elsif ( ref($output) ) {
+        _hash2xml2fh( $output, $hash, @options{qw( root version encoding indent canonical use_attr)} );
     }
     else {
-        die "Invalid output type: '".ref($output)."'";
+        die "Invalid output type";
     }
 }
 
@@ -65,7 +66,8 @@ $hash is reference to hash
             node4 => sub { return 'value4' },
             node5 => sub { return { node51 => 'value51' } },
         },
-        indent => 1
+        canonical => 1,
+        indent    => 2,
     ;
 
 will convert to:
@@ -73,11 +75,9 @@ will convert to:
     <?xml version="1.0" encoding="utf-8"?>
     <root>
       <node1>value1</node1>
+      <node2>value21</node2>
       <node2>
-        <item>value21</item>
-        <item>
-          <node22>value22</node22>
-        </item>
+        <node22>value22</node22>
       </node2>
       <node3>value3</node3>
       <node4>value4</node4>
@@ -86,6 +86,31 @@ will convert to:
       </node5>
     </root>
 
+and (use_attr=1):
+
+    hash2xml
+        {
+            node1 => 'value1',
+            node2 => [ 'value21', { node22 => 'value22' } ],
+            node3 => \'value3',
+            node4 => sub { return 'value4' },
+            node5 => sub { return { node51 => 'value51' } },
+        },
+        use_attr  => 1,
+        canonical => 1,
+        indent    => 2,
+    ;
+
+will convert to:
+
+    <?xml version="1.0" encoding="utf-8"?>
+    <root node1="value1" node3="value3" node4="value4">
+      <node2>value21</node2>
+      <node2 node22="value22"/>
+      <node5 node51="value51"/>
+    </root>
+
+
 =head1 OPTIONS
 
 =over 4
@@ -104,7 +129,8 @@ XML output encoding
 
 =item indent [ = 0 ]
 
-if idnent is "1", XML output should be indented according to its hierarchic structure.
+if indent great than "0", XML output should be indented according to its hierarchic structure.
+This value determines the number of spaces.
 
 if indent is "0", XML output will all be on one line.
 
@@ -116,6 +142,18 @@ if output is undefined, XML document dumped into string.
 
 if output is FH, XML document writes directly to a filehandle or a stream.
 
+=item canonical [ = 0 ]
+
+if canonical is "1", converter will be write hashes sorted by key.
+
+if canonical is "0", order of the element will be pseudo-randomly.
+
+=item use_attr [ = 0 ]
+
+if use_attr is "1", converter will be use the attributes.
+
+if use_attr is "0", converter will be use tags only.
+
 =back
 
 =head1 AUTHOR
